﻿using Discord;
using Discord.Commands;
using Discord.WebSocket;
using Microsoft.Extensions.DependencyInjection;
using Sean.Services;
using System;
using System.Threading.Tasks;

namespace Sean
{
    internal class Program
    {
        #region Fields
        private DiscordSocketClient client;
        private CommandHandler commandHandler;
        private CommandService commandService;
        private CommandServiceConfig commandServiceConfig;
        #endregion Fields

        #region Methods
        //.AddSingleton<DatabaseService>()
        public IServiceProvider BuildProvider() => new ServiceCollection().BuildServiceProvider();
        public IServiceProvider BuildServiceProvider() => new ServiceCollection()
                    .AddSingleton(client)
                    .AddSingleton(commandService)
                    .AddSingleton<CommandHandler>()
                    .AddSingleton<AwardsPersistencyService>()
                    .BuildServiceProvider();
        public async Task MainAsync()
        {
            commandServiceConfig = new CommandServiceConfig
            {
                CaseSensitiveCommands = false,
                LogLevel = LogSeverity.Debug
            };

            commandService = new CommandService(commandServiceConfig);
            client = new DiscordSocketClient();
            client.Log += Log;

            IServiceProvider provider = BuildServiceProvider();

            commandHandler = new CommandHandler(provider, client, commandService);
            await commandHandler.InstallCommandsAsync();

            await client.LoginAsync(TokenType.Bot, Environment.GetEnvironmentVariable("DiscordToken_Sean"));
            await client.StartAsync();

            // Block this task until the program is closed.
            await Task.Delay(-1);
        }
        private static void Main(string[] args)
                        => new Program().MainAsync().GetAwaiter().GetResult();
        private Task Log(LogMessage msg)
        {
            Console.WriteLine(msg.ToString());
            return Task.CompletedTask;
        }
        #endregion Methods
    }
}